<?php
/**
 * Created by PhpStorm.
 * User: tuliomagalhaes
 * Date: 17/04/2015
 * Time: 10:40
 */

namespace Service;

use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Util\Utilities;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\Framework\Remote\FileUploader\Server;

class Attachment {

    protected $directoryBasicUpload;

    private $entityManager;
    private $environment;
    private $fileServerUrl;
    private $httpKernel;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Comments  */
    protected $commentsRepository;
    /** @var Server  */
    private $zhFileUploader;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Attachment  */
    private $attachmentRepository;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Task  */
    private $taskRepository;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Project  */
    private $projectRepository;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\TaskRelationship  */
    private $taskRelationshipRepository;

    public function __construct(EntityManager $entityManager, \Util\Utilities $environment, \Factory\Attachment $attachment,
                                Kernel $httpKernel,$attachmentsBaseDirectory,  Server $zhFileUploader, $fileServerUrl)
    {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->attachment = $attachment;
        $this->httpKernel = $httpKernel;
        $this->attachmentRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'Attachment');
        $this->taskRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'Task');
        $this->commentsRepository  = $entityManager->getRepository(Utilities::ENTITY_DIR.'Comments');
        $this->projectRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'Project');
        $this->taskRelationshipRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'TaskRelationship');
        $this->directoryBasicUpload = $attachmentsBaseDirectory . DIRECTORY_SEPARATOR . 'attachments';
        $this->zhFileUploader    = $zhFileUploader;
        $this->fileServerUrl = $fileServerUrl;
    }


    public function getDataSetForEditAttachment($id, $entity, $conditions) {
        $return = [];
        if($entity == 'task'){
            /** @var Entities\Task $task */
            $task = $this->taskRepository->find($id);
            if($task){
                $isIssue = $task->isIssue();
                /** @var Entities\TaskRelationship $firstTaskRelationship */
                $firstTaskRelationship = $this->taskRelationshipRepository->findOneBy(array('task' => $id,
                    'type' => Entities\TaskRelationship::FIRST_TYPE));
                if($firstTaskRelationship){
                    $id = $firstTaskRelationship->getTaskRelationship()->getId();
                }
                if($firstTaskRelationship){
                    $id = $firstTaskRelationship->getTaskRelationship()->getId();
                }else{
                    $isIssue = true;
                }
                $return = $this->attachmentRepository->getAttachmentFromTask($id, $conditions, $isIssue);
            }
        }else{
            $return = $this->attachmentRepository->getDataSetForEditAttachment($id, $entity, $conditions);
        }
        return $return;
    }

    public function addDocumentsTask($taskId,array $documents, $commentId = null){
        
        $task = $this->taskRepository->find($taskId);
        $filesUploaded = $this->updateDocumentTask($documents, $task, $commentId);
        $this->entityManager->flush();
        return $filesUploaded;
    }


    public function addDocumentComment($commentId, array $documents){
        $comment  = $this->commentsRepository->find($commentId);
        $filesUploaded = $this->updateDocumentComment($documents, $comment);
        $this->entityManager->flush();
        return $filesUploaded;
    }

    public function saveLog($taskId,array $documents){
        $task = $this->taskRepository->find($taskId);
        $filesUploaded = $this->updateLog($documents, $task);
        $this->entityManager->flush();
        return $filesUploaded;
    }

    public function getFullLog($labelFile){
        $fileContentString = '';
        /** @var Entities\Attachment[] $oldLogFiles */
        $oldLogFiles = $this->attachmentRepository->findBy(array("label" => $labelFile));
        foreach($oldLogFiles as $oldLogFile){
            try {
                $file = file_get_contents($this->fileServerUrl . $oldLogFile->getFileName());
                $fileContentString .= $file;
            }catch (\Exception $e){}
        }
        return $fileContentString;
    }

    public function addDocumentsProject($projectId,array $documents){
        $project = $this->projectRepository->find($projectId);
        $this->updateDocumentProject($documents, $project);
        $this->entityManager->flush();
    }

    public function downLoadAttachment($attachmentId)
    {
        /**@var Entities\Attachment $attachment */
        $attachment = $this->attachmentRepository->find($attachmentId);

        $name = $attachment->getLabel();
        $size = $attachment->getSizeFile();
        header('Accept-Ranges: none');
        header("Content-Type: " . $attachment->getTypeFile()); // informa o tipo do arquivo ao navegador
        header("Content-Length: " . $size); // informa o tamanho do arquivo ao navegador
        header('Content-Disposition: attachment; filename="' . $name.'"'); // informa ao navegador que é tipo anexo e faz abrir a janela de download, tambem informa o nome do arquivo
        if($attachment->getFileName()[0] != '.') {
            readfile($this->fileServerUrl . $attachment->getFileName());
        }else{
            readfile($attachment->getFileName());
        }
        exit;
    }
    public function deleteAttachment($rows){
        /**
         * @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Attachment $attachment
         */
        foreach($rows as $row){
            $attachment = $this->attachmentRepository->find($row['ID']);
            try {
                if($this->verifyMultipleUses($attachment->getFileName())) {
                    $this->zhFileUploader->deleteFile($attachment->getFileName());
                }
            } catch (\Exception $e){
                // If something goes wrong with deleteFile(), we should do nothing, only remove it from database
            }
            $this->entityManager->remove($attachment);
        }
        $this->entityManager->flush();
    }

    private function verifyMultipleUses($filePath){
        $relatedAttachments = $this->attachmentRepository->findBy(array('fileName' => $filePath));
        return count($relatedAttachments) == 1;
    }

    public function deleteAttachmentByEntities(array $attachments){
        /** @var Entities\Attachment[] $attachments */
        foreach($attachments as $attachment){
            try {
                $this->zhFileUploader->deleteFile($attachment->getFileName());
            } catch (\Exception $e){
                // If something goes wrong with deleteFile(), we should do nothing, only remove it from database
            }
            $this->entityManager->remove($attachment);
        }
        $this->entityManager->flush();
    }

    public function updateDocumentTask(array $documents,\Zeedhi\DevOps\Model\Projectmanagement\Entities\Task $task, $commentId = null){
        $filesUploaded = array();
        foreach($documents as $fileData) {

            $typeFile = $fileData['type'];
            $labelFile = $fileData['name'];
            $sizeFile = $fileData['size'];
            $pathFile = $fileData['path'];
            if($commentId)
            {
                $comentario = InstanceProvider::getRepository()->getComments()->findOneBy(array('id'=>$commentId));
            }else{
                $comentario = null;
            }
            $attachment= $this->attachment->createAttachmentTask($typeFile,$pathFile,$labelFile,$sizeFile,$task,$comentario);
            $this->entityManager->persist($attachment);
            $filesUploaded[] = $attachment;
        }
        $this->entityManager->flush();
        return $filesUploaded;
    }

    public function updateDocumentComment(array $documents,\Zeedhi\DevOps\Model\Projectmanagement\Entities\Comments $comments){
        $filesUploaded = array();
        foreach($documents as $fileData) {
            $typeFile = $fileData['type'];
            $labelFile = $fileData['name'];
            $sizeFile = $fileData['size'];
            $pathFile = $fileData['path'];
            $attachment= $this->attachment->createAttachmentComment($typeFile,$pathFile,$labelFile,$sizeFile,$comments);
            $this->entityManager->persist($attachment);
            $filesUploaded[] = $attachment;
        }
        $this->entityManager->flush();
        return $filesUploaded;
    }

    public function updateLog(array $documents,\Zeedhi\DevOps\Model\Projectmanagement\Entities\Task $task){
        $filesUploaded = array();
        foreach($documents as $fileData) {
            $labelFile = $fileData['name'];
            /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Attachment $oldLogFile  */
            $oldLogFile = $this->attachmentRepository->findOneBy(array("label" => $labelFile));
            $typeFile = $fileData['type'];
            $sizeFile = $fileData['size'];
            $pathFile = $fileData['path'];
            if(is_null($oldLogFile)) {
                $attachment = $this->attachment->createAttachmentTask($typeFile, $pathFile, $labelFile, $sizeFile, $task);
            }else{
                $this->zhFileUploader->deleteFile($oldLogFile->getFileName());
                $attachment = $this->attachment->updateAttachmentPath($oldLogFile, $pathFile);
            }
            $this->entityManager->persist($attachment);
            $filesUploaded[] = $attachment;
        }
        $this->entityManager->flush();
        return $filesUploaded;
    }
    public function updateDocumentProject(array $documents,\Zeedhi\DevOps\Model\Projectmanagement\Entities\Project $project){
        $filesUploaded = array();
        foreach($documents as $fileData) {
            $typeFile = $fileData['type'];
            $labelFile = $fileData['name'];
            $sizeFile = $fileData['size'];
            $pathFile = $fileData['path'];
            $attachment= $this->attachment->createAttachmentProject($typeFile,$pathFile,$labelFile,$sizeFile,$project);
            $this->entityManager->persist($attachment);
        }
        $this->entityManager->flush();
        return $filesUploaded;
    }
    /**
     * @param $basic
     * @param \Zeedhi\DevOps\Model\Projectmanagement\Entities\Task $task
     * @param \Zeedhi\DevOps\Model\Projectmanagement\Entities\Attachment[] $attachments
     * @param $move
     */
    public function attachmentsMoveOrCopy($basic,\Zeedhi\DevOps\Model\Projectmanagement\Entities\Task $task, array $attachments ,$move = false)
    {

        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Attachment $attachment */

        if(!empty($attachments)){
            foreach ($attachments as $attachment) {
                if($move){
                    $attachment->setProject($task->getProject());
                    $this->entityManager->persist($attachment);
                }else{
                    $newAttachment = $this->attachment->createAttachmentTask($attachment->getTypeFile(),$attachment->getFileName(),$attachment->getLabel(),
                        $attachment->getSizeFile(),$task);
                    $this->entityManager->persist($newAttachment);
                }
            }
        }
    }
    public function projectAttachmentsMoveOrCopy(Entities\Project $project, array $attachments ,$move = false)
    {

        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Attachment $attachment */

        if(!empty($attachments)){
            foreach ($attachments as $attachment) {
                if($move){
                    $attachment->setProject($project);
                    $this->entityManager->persist($attachment);
                }else{
                    $newAttachment = $this->attachment->createAttachmentProject($attachment->getTypeFile(),$attachment->getFileName(),$attachment->getLabel(),
                        $attachment->getSizeFile(),$project);
                    $this->entityManager->persist($newAttachment);
                }
            }
        }
    }

    public function getIssueAttachmentsExceptUploadedBy($issueId) {
        return $this->attachmentRepository->getTaskAttachmentsExceptUploadedBy($issueId,$this->fileServerUrl);
    }

    public function getIssueById($id) {
        return $this->attachmentRepository->getIssueById($id,$this->fileServerUrl);
    }
    
    public function checkIfRelatedTasksHasSameAttachment($attachmentId)
    {
        $attachment = $this->attachmentRepository->find($attachmentId);
        $taskId = $attachment->getTask()? $attachment->getTask()->getId() : 0;
        $relatedTaskIds = $this->taskRelationshipRepository->findRelatedTaskIds($taskId);
        $attachmentIds = array();
        if(!empty($relatedTaskIds)){
            $allRelatedTasksIds = explode(",", $relatedTaskIds["RELATED_TASKS"]);
            array_push($allRelatedTasksIds, $relatedTaskIds["MAIN_TASK"]);
            $attachments = $this->attachmentRepository->findBy(array("task" => $allRelatedTasksIds, "fileName" => $attachment->getFileName()));
            $attachmentIds = Utilities::getArrayByArrayObjectWithGet($attachments);
        } 
        return $attachmentIds;
    }

    public function createDocuments(array $documents){
        $filesUploaded = array();
        foreach($documents as $fileData) {
            $typeFile = $fileData['type'];
            $labelFile = $fileData['name'];
            $sizeFile = $fileData['size'];
            $pathFile = $fileData['path'];
            $attachment = $this->attachment->createAttachment($typeFile,$pathFile,$labelFile,$sizeFile);
            $this->entityManager->persist($attachment);
            $filesUploaded[] = $attachment;
        }
        $this->entityManager->flush();
        return $filesUploaded;
    }

} 