<?php
/**
 * Created by PhpStorm.
 * User: eugeniofonseca
 */

namespace Service;


use Util\InstanceProvider;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories\BudgetPlots;
use Zeedhi\DevOps\Util\Utilities;

class Budget {

    public function budgetItem($conditions,$page,$pageSize,$notClassificationId = null) {
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $conditionsSQL = Utilities::createQueryFiltered($conditions);
        $budgetItems = InstanceProvider::getRepository()->getBudgetItem()->budgetItem(
            InstanceProvider::getEnvironment()->getCurrentOrganizationId(), $conditionsSQL, $limit,
            $offset,$notClassificationId);
        return $budgetItems;
    }

    public function findIssueBudget($issueId,$nameProposal,$nameDiscount,$namePercent)
    {
        return InstanceProvider::getRepository()->getBudget()
            ->findIssueBudget($issueId,$nameProposal,$nameDiscount,$namePercent);
    }

    public function findMaintenanceCost($issueId)
    {
        $budgetRepository = InstanceProvider::getRepository()->getBudget();
        $organizationId = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        $maintenance = $budgetRepository->MaintenanceCost($organizationId);
        if($maintenance){
            $budgetPrice = $budgetRepository->findManutIssueBudget($issueId);
            $finalPrice = round($budgetPrice[0]["TOTAL_VALUE"], 2);
            $maintenancePrice = round($finalPrice * $maintenance / 100, 2);
            $maintenancePrice = number_format($maintenancePrice, 2, '.', '');

            $maintenanceDiscounts = $budgetRepository->findManutIssueDiscount($issueId);
            if($maintenanceDiscounts && $maintenanceDiscounts[0]){
                $result['VALUE_DISCOUNT'] = $maintenanceDiscounts[0]['MAINTENANCE_VALUE'];
                $result['PERCENTAGE_DISCOUNT'] =  $maintenanceDiscounts[0]['MAINTENANCE_PERCENT'];
                $result['TOTAL_VALUE'] = $maintenancePrice - $maintenanceDiscounts[0]['MAINTENANCE_VALUE'];
            }

            $result['MANUT_VALUE'] = $maintenancePrice;
        }else{
            $result['MANUT_VALUE'] = null;
            $result['TOTAL_VALUE'] = null;
            return $result;
        }


        return $result;
    }

    public function findIssueBudgetPlots($issueId,$namePlots)
    {
        return InstanceProvider::getRepository()->getBudget()
            ->findIssueBudgetPlots($issueId,$namePlots);
    }


    public function saveMaintenanceCost($row)
    {
      $entityManager = InstanceProvider::getEntityManager();
      $connection = $entityManager->getConnection();
        try {
            $connection->beginTransaction();
            if ($row['ISSUE_ID']) {
                $issue = InstanceProvider::getRepository()->getTask()->find($row['ISSUE_ID']);
                /** @var Entities\Budget $budget */
                $budget = InstanceProvider::getRepository()->getBudget()->findOneBy(array('task' => $issue));

                $valueDiscount = str_replace(",", ".", $row['VALUE_DISCOUNT']);
                $valueManut = str_replace(",", ".", $row['MANUT_VALUE']);
                $valueDiscount = floatval(preg_replace('/[^\d\.]+/', '', $valueDiscount));
                $valuePercent = str_replace(",", ".", $row['PERCENTAGE_DISCOUNT']);
                if($valueManut < $valueDiscount){
                    $valueDiscount = $valueManut;
                }
                if($valuePercent > 100){
                    $valuePercent = 100;
                }
                $budget->setMaintenancePercent($valuePercent);
                $budget->setMaintenanceValue($valueDiscount);

                $entityManager->flush();
                $connection->commit();

            }
        }
        catch(\Exception $e){
            $connection->rollBack();
            throw $e;
        }



    }

    public function issueBudgetSave($rows){

        $entityManager = InstanceProvider::getEntityManager();
        $instanceRepository = InstanceProvider::getRepository();
        $connection = $entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $budgetId = null;
            foreach ($rows as $row) {
                $issueId = $row['ISSUE_ID'];
                $budgetItemId = $row['BUDGET_ITEM_ID'];
                $proposalQuantity = $row['PROPOSAL_QUANTITY'];
                $proposalValue = $row['PROPOSAL_VALUE'];
                if(!is_numeric($proposalValue)){
                    $proposalValue = str_replace(',', '.', $proposalValue);
                    $proposalValue = floatval($proposalValue);
                }
                $budgetId = InstanceProvider::getService()->getBudget()->issueBudgetRowSave($issueId, $budgetItemId,
                    $proposalQuantity, $proposalValue);
            }

            $budget = InstanceProvider::getRepository()->getBudget()->findOneBy(array('task' => $issueId));

            if($budget && $budget->getMaintenancegetPercent()){
                $budgetRepository = InstanceProvider::getRepository()->getBudget();
                $organizationId = InstanceProvider::getEnvironment()->getCurrentOrganizationId();

                $budgetPrice = $budgetRepository->findManutIssueBudget($issueId);
                $maintenance = $budgetRepository->MaintenanceCost($organizationId);
                $finalPrice = round($budgetPrice[0]["TOTAL_VALUE"] , 2);
                $maintenancePrice = round($finalPrice * $maintenance / 100, 2);
                $maintenancePrice = number_format($maintenancePrice, 2);

                if($maintenancePrice > 0){
                    $discountValue = $budget->getMaintenancegetPercent() * $maintenancePrice / 100;

                    $budget->setMaintenanceValue($discountValue);

                }else{
                    $budget->setMaintenanceValue(0);
                }
                $entityManager->flush();
            }
            $repositoryBudgetPlots = $instanceRepository->getBudgetPlots();
            $budgetPlotsToDelete = $repositoryBudgetPlots->findBy(array('budget' => $budgetId));
            foreach ($budgetPlotsToDelete as $parcelToDelete) {
                $entityManager->remove($parcelToDelete);
            }
            if(!empty($budgetPlotsToDelete)){
                $entityManager->flush();
            }
            $connection->commit();
        }
        catch(\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }

    public function redoBudget($issueId, $specificationComment)
    {
        $budgetRepository = InstanceProvider::getRepository()->getBudget();
        /** @var Entities\Budget $budget */
        $budget = $budgetRepository->findOneBy(array('task' => $issueId));
        if ($budget ) {
            $organizationId = \Util\Utilities::getEnvironment()->getCurrentOrganizationId();
            $urlSm = $budgetRepository->getUrlIntegrationSM($organizationId);
            if ($urlSm) {
                if ($budget->getExternalId()) {
                    Integration\IntegrationSM::integrationSMCancel($issueId, $organizationId);
                }
                /** @var Entities\Comments $comment */
                $comment = InstanceProvider::getRepository()->getComments()->findOneBy(array('task' => $issueId, 'commentType' => 3), array('createdAt' => 'DESC'));
                $attachments = InstanceProvider::getRepository()->getAttachment()->findBy(array('comments' => $comment));
                if(empty($attachments)){
                    throw \Exception('Não foi possível refazer a proposta porque o documento da última especificação foi removida!',10);
                }
                $cdOperador = \Util\Utilities::getEnvironment()->getCurrentCdoperador();
                $entityManager = InstanceProvider::getEntityManager();
                $connection = $entityManager->getConnection();
                try {
                    $connection->beginTransaction();

                    \Util\Utilities::getTaskService()->createdTaskClientAccept($budget->getTask(),$budget->getTask()->getProject());

                    $commentNew = $this->createCommentEspecification($specificationComment, $budget, $comment);
                    $this->moveAttachments($attachments, $commentNew, $entityManager);

                    $proposalId = Integration\IntegrationSM::integrationSMFinishSpecification($issueId, $organizationId);
                    $this->createVedendor($cdOperador, $proposalId, $organizationId, $entityManager);
                    $budget->setExternalId($proposalId);
                    $entityManager->persist($budget);
                    $entityManager->flush();
                    $connection->commit();
                } catch (\Exception $e) {
                    $connection->rollBack();
                    throw $e;
                }
            }
        }
    }

    public function allowRedoBudget($issueId){
        $return = false;
        $budgetRepository = InstanceProvider::getRepository()->getBudget();
        /** @var Entities\Budget $budget */
        $budget = $budgetRepository->findOneBy(array('task' => $issueId));
        if($budget){
            $organizationId = \Util\Utilities::getEnvironment()->getCurrentOrganizationId();
            $urlSm = $budgetRepository->getUrlIntegrationSM($organizationId);
            if($urlSm &&
                $budget->getTask()->getTaskStatus() &&
                $budget->getTask()->getTaskStatus()->getControl() == Task::TASK_STATUS_CONTROL_BACKLOG){

                /** @var Entities\Comments $comment */
                $comment = InstanceProvider::getRepository()->getComments()->findOneBy(array('task' => $issueId ,'commentType' => 3),array('createdAt'=> 'DESC'));
                if($comment){
                    $classificationId = $budget->getTask()->getClassification()->getId();
                    $taskSprint = InstanceProvider::getRepository()->getTask()->getTaskRelationshipByIssue($issueId,  " AND CLASSIFICATION_ID = $classificationId AND PERCENT <> 100");
                    if(!empty($taskSprint)){
                        $return = true;
                    }
                }
            }
        }
        return $return;
    }

    public function issueBudgetPlotsSave($rows){

        $connection = InstanceProvider::getEntityManager()->getConnection();
        try {
            $connection->beginTransaction();

            $instanceRepository = InstanceProvider::getRepository();
            $entityManager = InstanceProvider::getEntityManager();
            $factoryBudgetPlots = InstanceProvider::getFactory()->getBudgetPlots();
            /** @var BudgetPlots $repositoryBudgetPlots */
            $repositoryBudgetPlots = $instanceRepository->getBudgetPlots();

            /** @var Entities\Budget $budget */
            $budget = $instanceRepository->getBudget()->find($rows[0]['BUDGET_ID']);
            /** @var Entities\BudgetPlots[] $budgetPlots */
            //$budgetPlots = $instanceRepository->getBudgetPlots()->findby(array('budget' => $budget),array('orderPlots' => 'ASC'));
            $lastParcel = 0;

            foreach ($rows as $row) {
                $orderPlots = intval($row['ORDER_PLOTS_NUM']);
                if($lastParcel < $orderPlots){
                    $lastParcel = $orderPlots;
                }
                $valuePlots = $row['VALUE_PLOTS'];
                $daysPayment = $row['DAYS_PAYMENT'];
                /** @var Entities\BudgetPlots $budgetPlots */
                $budgetPlots = $repositoryBudgetPlots->findOneBy(array('budget' => $budget,'orderPlots' => $orderPlots));
                if($budgetPlots){
                    $budgetPlots = $factoryBudgetPlots->save($valuePlots,$daysPayment,$budgetPlots);
                }else{
                    $budgetPlots = $factoryBudgetPlots->save($valuePlots,$daysPayment,null,$budget,$orderPlots);
                }
                $entityManager->persist($budgetPlots);
            }
            $entityManager->flush();

            if($lastParcel){
                $budgetPlotsIdToDelete = $repositoryBudgetPlots->findBudgetPlotsToDelete($budget->getId(),$lastParcel);
                $budgetPlotsToDelete = $repositoryBudgetPlots->findBy(array('id' => $budgetPlotsIdToDelete));
                foreach ($budgetPlotsToDelete as $parcelToDelete) {
                    $entityManager->remove($parcelToDelete);
                }
                if(!empty($budgetPlotsToDelete)){
                    $entityManager->flush();
                }
            }
            $connection->commit();
        }
        catch(\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }

    public function issueBudgetRowSave($issueId,$budgetItemId,$proposalQuantity,$proposalValue)
    {

        $entityManager = InstanceProvider::getEntityManager();
        $instanceRepository = InstanceProvider::getRepository();
        $instanceFactory = InstanceProvider::getFactory();
        /** @var Entities\Task $issue */
        $issue = $instanceRepository->getTask()->find($issueId);
        /** @var Entities\Budget $budget */
        $budget = $instanceRepository->getBudget()->findOneBy(array('task' => $issue));
        $budget = $instanceFactory->getBudget()->save($budget,$issue);
        if($budgetItemId == 'XXXX'){
            $proposalQuantity = abs($proposalQuantity);
            if($proposalQuantity > 100){
                $proposalQuantity = 100;
            }
            $budget->setDiscountPercent($proposalQuantity);
            $budget->setDiscountValue($proposalValue);
            $entityManager->persist($budget);
            $entityManager->flush();
        }else{
            /** @var Entities\BudgetItem $budgetItem */
            $budgetItem = $instanceRepository->getBudgetItem()->find($budgetItemId);
            /** @var Entities\BudgetItemBudget $budgetItemBudget */
            $budgetItemBudget = $instanceRepository->getBudgetItemBudget()
                ->findOneBy(array( 'budgetItem' => $budgetItem, 'budget' => $budget, ));
            if($proposalQuantity){
                $valueItem = $budgetItem->getValue();
                $unitMeasure = $budgetItem->getUnitMeasure();
                $budgetItemBudget = $instanceFactory->getBudgetItemBudget()->save($budgetItemBudget,$budget,$budgetItem,
                    $proposalQuantity,$valueItem,$unitMeasure);
                $entityManager->persist($budgetItemBudget);
                $entityManager->flush();
            }else if($budgetItemBudget){
                $entityManager->remove($budgetItemBudget);
                $entityManager->flush();
            }
        }
        return $budget->getId();
    }

    public function classificationBudgetItem($classificationId,$conditions,$page,$pageSize) {
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $conditionsSQL = Utilities::createQueryFiltered($conditions);
        $budgetItems = InstanceProvider::getRepository()->getClassificationBudgetItem()->classificationBudgetItem($classificationId,
            $conditionsSQL, $limit, $offset);
        return $budgetItems;
    }

    public function budgetItemSave($id,$name,$valueItem,$unitMeasure,$status) {
        $entityManager = InstanceProvider::getEntityManager();
        $budgetItem = null;
        if($id){
            /** @var Entities\BudgetItem $budgetItem */
            $budgetItem = InstanceProvider::getRepository()->getBudgetItem()->find($id);
        }
        $valueItem = Utilities::fixBugSaveCurrencyMask($valueItem);
        $budgetItem = InstanceProvider::getFactory()->getBudgetItem()->save($budgetItem,$name, $valueItem, $unitMeasure,
            $status);
        $entityManager->persist($budgetItem);
        $entityManager->flush();
        return ['ID' => $budgetItem->getId()];
    }

    public function addBudgetItemInClassificationSave($budgetItemId,$classificationId) {
        $entityManager = InstanceProvider::getEntityManager();
        $classificationBudgetItemId = null;
        if($budgetItemId && $classificationId){
            /** @var Entities\BudgetItem $budgetItem */
            $budgetItem = InstanceProvider::getRepository()->getBudgetItem()->find($budgetItemId);
            /** @var Entities\Classification $classification */
            $classification = InstanceProvider::getRepository()->getClassification()->find($classificationId);
            $classificationBudgetItem = InstanceProvider::getRepository()->getClassificationBudgetItem()
                ->findOneBy(array('classification' => $classificationId,'budgetItem' => $budgetItemId,));
            if(!$classificationBudgetItem){
                $classificationBudgetItem = InstanceProvider::getFactory()->getclassificationBudgetItem()->create($budgetItem,$classification);

                $entityManager->persist($classificationBudgetItem);
                $entityManager->flush();
                $classificationBudgetItemId = $classificationBudgetItem->getId();
            }
        }
        return ['ID' => $classificationBudgetItemId];
    }

    public function budgetItemRemove($id) {
        $entityManager = InstanceProvider::getEntityManager();
        /** @var Entities\BudgetItem $budgetItem */
        $budgetItem = InstanceProvider::getRepository()->getBudgetItem()->find($id);
        $budgetItemBudget = InstanceProvider::getRepository()->getBudgetItemBudget()->findOneBy(array('budgetItem' => $budgetItem));

        if(!$budgetItemBudget){
            $classificationBudgetItems = InstanceProvider::getRepository()->getClassificationBudgetItem()
                ->findBy(array('budgetItem' => $budgetItem));

            foreach ($classificationBudgetItems as $classificationBudgetItem) {
                $entityManager->remove($classificationBudgetItem);
            }
            $entityManager->flush();

            $budgetItem->deleteEntity();
        }else{
            $budgetItem->setStatus(Entities\BudgetItem::IS_NOT_ACTIVE);
        }
        $entityManager->persist($budgetItem);
        $entityManager->flush();
    }

    public function classificationBudgetItemRemove($id) {
        $entityManager = InstanceProvider::getEntityManager();
        /** @var Entities\ClassificationBudgetItem $budgetItem */
        $classificationBudgetItem = InstanceProvider::getRepository()->getClassificationBudgetItem()->find($id);
        $entityManager->remove($classificationBudgetItem);
        $entityManager->flush();
    }

    /**
     * @param $specificationComment
     * @param Entities\Budget $budget
     * @param Entities\Comments $comment
     * @return Entities\Comments
     */
    protected function createCommentEspecification($specificationComment, Entities\Budget $budget, Entities\Comments $comment): Entities\Comments
    {
        $commentNew = InstanceProvider::getService()->getComment()->commentInSpecification($budget->getTask(),
            $specificationComment, $comment->getCommentType());
        return $commentNew;
    }

    /**
     * @param array $attachments
     * @param Entities\Comments $commentNew
     * @param \Doctrine\ORM\EntityManager $entityManager
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    protected function moveAttachments(array $attachments, Entities\Comments $commentNew, \Doctrine\ORM\EntityManager $entityManager): void
    {
        /** @var Entities\Attachment[] $attachments */
        foreach ($attachments as $attachment) {
            $attachment->setComments($commentNew);
            $entityManager->persist($attachment);
        }
        $entityManager->flush();
    }

    /**
     * @param $cdOperador
     * @param $proposalId
     * @param int $organizationId
     * @param \Doctrine\ORM\EntityManager $entityManager
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    protected function createVedendor($cdOperador, $proposalId, int $organizationId, \Doctrine\ORM\EntityManager $entityManager): void
    {
        $nrparcnegocio = InstanceProvider::getRepository()->getTaskRelationship()->findNrparcnegocioByCdoperator($cdOperador);
        if (!empty($nrparcnegocio) && $proposalId) {
            $nrparcnegocio = $nrparcnegocio[0]['NRPARCNEGOCIO'];
            $vendedorProposta = InstanceProvider::getFactory()->getFctVendedororca()->save(null, $proposalId, $nrparcnegocio,
                $organizationId, $cdOperador, $cdOperador);
            $entityManager->persist($vendedorProposta);
            $entityManager->flush();
        }
    }
}
