<?php

namespace Service;

use Doctrine\ORM\EntityManager;
use Util\Messages;
use Util\Utilities;
use Zeedhi\Framework\Controller\Exception;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;


class Repository
{

    protected $entityManager;
    protected $environment;
    protected $repositoryFactory;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Repository */
    protected $repositoryRepository;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Project */
    protected $projectRepository;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Directory */
    protected $directoryRepository;
    /** @var \Zeedhi\DevOps\Model\Configurationmanager\Repositories\Product */
    protected $productRepository;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Task */
    protected $taskRepository;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\TaskStatus */
    protected $taskStatusRepository;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\ChangedFiles */
    protected $changedFilesRepository;

    public function __construct(EntityManager $entityManager, \Util\Utilities $environment, \Factory\Repository $repositoryFactory)
    {

        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->repositoryFactory = $repositoryFactory;
        $this->repositoryRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Repository');
        $this->directoryRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Directory');
        $this->projectRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Project');
        $this->taskRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Task');
        $this->taskStatusRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'TaskStatus');
        $this->changedFilesRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'ChangedFiles');
        $this->productRepository = $entityManager->getRepository(Utilities::CONFIGURATION_ENTITY_DIR . 'Product');
        \Util\InstanceProvider::getLog()->setLog('instanciando Repository service',false,2);
    }

    public function getAllRepositories($project = null, $filter = array())
    {
        $filterString = Utilities::createQueryFiltered($filter);
        $repositories = $this->repositoryRepository->findAllRepositories($this->environment->getCurrentOrganizationId(),$filterString);
        if (!empty($project)) {
            $alreadyAssociatedRepositories = $this->findProjectRepositories($project, '');
            foreach ($alreadyAssociatedRepositories as $key => $repository) {
                $alreadyAssociatedRepositories[$key] = $repository['URL'];
            }
            foreach ($repositories as $key => $repository) {
                if (in_array($repository['URL'], $alreadyAssociatedRepositories)) {
                    unset($repositories[$key]);
                }
            }
        }
        $repositories = array_values($repositories);
        return $repositories;
    }

    //Methods from projectRepository:
    public function findResourceProjectRepositories($resource, $project)
    {
        if ($resource == 0) {
            $resource = $this->environment->getCurrentUserId();
        }
        return $Repository = $this->repositoryRepository->findRepositoriesByResource($resource, $project);
    } // Fixed 22/05


    public function getRepositoriesByTaskId($taskId)
    {
        $resourceId = $this->environment->getCurrentUserId();
        return $this->repositoryRepository->getRepositoriesByTaskId($taskId,$resourceId);
    }

    protected function persistRepository($repository, $status, $url, $description, $repositoryId, $project, $oldUrl,
                                         $product = null, $path = null,$seqno = null , $isMaster = 'N')
    {
        $organization = $this->environment->getCurrentOrganization();
        /** @var Entities\Repository $entity */
        $entity = $this->repositoryRepository->findOneBy(array('project' => $project->getId(), 'url' => $oldUrl));



        if (empty($entity)) {
            $description = empty($repositoryId) ? $description : $this->repositoryRepository
                ->find($repositoryId)->getDescription();
            $repository = $this->repositoryFactory->createRepository($organization, $repository, $description, $url,
                $status, $project, $product,$path ,$seqno );
        } else {
            if(!$path){
                $path = $entity->getPath();
            }
            if(!$seqno){
                $seqno = $entity->getSeqno();
            }
            if(!$isMaster){
                $isMaster = $entity->getIsMaster();
            }
            $repository = $this->repositoryFactory->updateRepository($entity, $repository, $description, $url, $status,
                $product,$path ,$seqno,$isMaster);
        }

        $this->entityManager->persist($repository);
        $this->entityManager->flush();
        $repositoryId = $repository->getId();

        return $repositoryId;
    }

    public function saveRepository($repository, $status, $url, $description, $repositoryId, $project,
                                   $path = null ,$seqno = null ,$isMaster = 'N')
    {
        /** @var Entities\Project $project */
        $project = $this->projectRepository->find($project);

        if ($repositoryId == 0) {
            $oldUrl = $url;
        } else {
            $oldUrl = $this->repositoryRepository->find($repositoryId)->getUrl();
        }
        $this->setRepositoryChildren($repository, $status, $url, $description, $repositoryId, $project, $oldUrl,
            $path, $seqno, $isMaster);

        $product = null;
        if ($project->getProjectType()->getExternaltype() == Project::PROJECT_TYPE_PRODUCT) {
            $product = $this->productRepository->findOneBy(array('project' => $project));
        }

        return $this->persistRepository($repository, $status, $url, $description, $repositoryId, $project,
            $oldUrl, $product, $path, $seqno, $isMaster);
    }

    public function findProjectRepositories($project, $filter){
        $children = $this->projectRepository->getProjectChildrenByIdForCount($project);
        $repositories = $this->repositoryRepository->getProjectRepositories($project, $filter);
        foreach($repositories as &$repository) {
            $repository["childProjects"] = count($children);
        }
        return $repositories;
    }

    public function findRepositoriesWithFilter($conditions){
        $filter = Utilities::createQueryFiltered($conditions, 'R');
        $repositories = $this->repositoryRepository->findRepositoriesWithFilter($filter);
        return $repositories;
    }

    public function findRepositories()
    {
        $repositories = $this->repositoryRepository->getRepositories();
        return $repositories;
    }

    public function saveProjectRepository(Entities\Project $project, $status, $description, $name, $url , $path = null ,$seqno = null)
    {
        $organization = $this->environment->getCurrentOrganization();
        $repositoryFactory = new \Factory\Repository($this->entityManager);

        $repository = $repositoryFactory->createRepository($organization, $name, $description, $url,
            $status, $project, null,$path ,$seqno);
        $this->entityManager->persist($repository);
        $this->entityManager->flush();

        return $repository;
    }

    public function deleteProjectRepositories($id, $project = null)
    {
        /** @var Entities\Repository $repository */
        $repository = $this->repositoryRepository->find($id);
        $this->validateDeleteRepository($repository); 
        $this->deleteRepositoriesChildren($project, $repository);
        $directories = $this->directoryRepository->findBy(array('repository' => $id));
        foreach ($directories as $directory) {
            $this->entityManager->remove($directory);
        }
        $repository->setStatus('D');
        $this->entityManager->persist($repository);
        $this->entityManager->flush();
    }

    public function verifyChangedFiles($repository, $task){
         return !!$this->changedFilesRepository->findOneBy(array('repository' => $repository,
                'task' => $task));
    }

    public function isInDeveloping($project, $developingStatus){
        return !!$this->taskRepository->findBy(array('project' => $project, 'taskStatus' => $developingStatus,
            'isIssue' => Entities\Task::NOT_IS_ISSUE));
    }

    /**
     * @param $repository
     * @param $status
     * @param $url
     * @param $description
     * @param $repositoryId
     * @param Entities\Project $project
     * @param $oldUrl
     * @param null $path
     * @param null $seqno
     * @param string $isMaster
     * @return null|object
     */
    protected function setRepositoryChildren($repository, $status, $url, $description, $repositoryId,
                                             Entities\Project $project, $oldUrl, $path = null ,$seqno = null ,$isMaster = 'N')
    {
        $childrenProjectIds = $this->projectRepository->getProjectChildrenNotConcludedOrCanceled($project->getId());
        /** @var  \Zeedhi\DevOps\Model\Projectmanagement\Entities\Project[] $childrenProjects */
        $childrenProjects = $this->projectRepository->findBy(array('id' => $childrenProjectIds));

        foreach ($childrenProjects as $childProject) {
            $product = null;
            if ($childProject->getProjectType()->getExternaltype() == Project::PROJECT_TYPE_PRODUCT) {
                $product = $this->productRepository->findOneBy(array('project' => $childProject));
            }
            $this->persistRepository($repository, $status, $url, $description, $repositoryId, $childProject, $oldUrl,
                $product, $path ,$seqno ,$isMaster);
        }
    }

    /**
     * @param $repository
     * @return mixed
     * @throws Exception
     */
    public function validateDeleteRepository(Entities\Repository $repository)
    {
        $project = $repository->getProject();
        $developingStatus = Utilities::getTaskStatusIsDeveloping();
        if ($this->isInDeveloping($project, $developingStatus)) {
            throw new Exception(Messages::ERR_REMOVE_REPOSITORY_TASK_IN_DEVELOPING);
        } else {
            $notInitializedStatus = Utilities::getTaskStatusNotInitialized();
            $pausedStatus = Utilities::getTaskStatusPaused();
            $tasks = $this->taskRepository->findBy(array('project' => $project,
                'taskStatus' => array($notInitializedStatus, $pausedStatus)));
            foreach ($tasks as $task) {
                if ($this->verifyChangedFiles($repository, $task)) {
                    throw new Exception(Messages::ERR_REMOVE_REPOSITORY_TASK_CHANGED_FILES);
                }
            }
        }
    }

    /**
     * @param $projectId
     * @param $repository
     */
    protected function deleteRepositoriesChildren($projectId, $repository)
    {
        if (!empty($projectId)) {
            /** @var Entities\Project[] $projectChildren */
            $projectChildren = $this->projectRepository->getProjectChildrenNotCanceledAndNotConcludedById($projectId);
            foreach ($projectChildren as $projectChild) {
                $repositoryChildren = $this->repositoryRepository->findOneBy(array('url' => $repository->getUrl(), 'project' => $projectChild->getId()));
                if (!empty($repositoryChildren)) {
                    $this->deleteProjectRepositories($repositoryChildren->getId());
                }
            }
        }
    }
}